/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal.model;

import com.redhat.thermostat.agent.utils.management.MXBeanConnection;

import javax.management.ObjectName;

public class JCMDBinding {

    private MXBeanConnection connection;

    public JCMDBinding(MXBeanConnection connection) {
        this.connection = connection;
    }

    public boolean canCollectDiagnosticData()throws Exception {
        String flags = getVmFlags();
        return flags.contains("+UnlockDiagnosticVMOptions");
    }

    public String getVmFlags() throws Exception {
        return invoke("vmFlags");
    }

    public String getGCStats() throws Exception {
        return invoke("gcClassStats",
                      new String[][] {
                              {
                                  "InstBytes,InstCount,KlassBytes,CpAll,"    +
                                  "annotations,MethodCount,MethodAll,ROAll," +
                                  "RWAll,ClassName,Bytecodes,ClassLoader,Total"
                              }

                      });
    }

    String invoke(String operation, String [][] params) throws Exception {

        return (String) connection.get().invoke(new ObjectName("com.sun.management:type=DiagnosticCommand"),
                                                operation, params,
                                                new String[] { String[].class.getName() });
    }

    String invoke(String operation) throws Exception {
        return invoke(operation, new String[][] { null });
    }
}
